@@ -6,7 +6,11 @@
 Represents an enumeration using a widget.
 """
 
-from collections import Mapping, Iterable
+try:
+    from collections.abc import Iterable, Mapping
+except ImportError:
+    from collections import Iterable, Mapping # py2
+
 try:
     from itertools import izip
 except ImportError:  #python3.x
@@ -26,16 +30,11 @@ from ipython_genutils.py3compat import unicode_type
 
 _doc_snippets = {}
 _doc_snippets['selection_params'] = """
-    options: list or dict
+    options: list
         The options for the dropdown. This can either be a list of values, e.g.
-        ``['Galileo', 'Brahe', 'Hubble']`` or ``[0, 1, 2]``, a list of
+        ``['Galileo', 'Brahe', 'Hubble']`` or ``[0, 1, 2]``, or a list of
         (label, value) pairs, e.g.
-        ``[('Galileo', 0), ('Brahe', 1), ('Hubble', 2)]``,
-        or a dictionary mapping the labels to the values, e.g. ``{'Galileo': 0,
-        'Brahe': 1, 'Hubble': 2}``. The labels are the strings that will be
-        displayed in the UI, representing the actual Python choices, and should
-        be unique. If this is a dictionary, the order in which they are
-        displayed is not guaranteed.
+        ``[('Galileo', 0), ('Brahe', 1), ('Hubble', 2)]``.
 
     index: int
         The index of the current selection.
@@ -113,12 +112,13 @@ def _make_options(x):
     The returned tuple should be in the format (('label', value), ('label', value), ...).
 
     The input can be
-    * a Mapping of labels to values
     * an iterable of (label, value) pairs
     * an iterable of values, and labels will be generated
     """
     # Check if x is a mapping of labels to values
     if isinstance(x, Mapping):
+        import warnings
+        warnings.warn("Support for mapping types has been deprecated and will be dropped in a future release.", DeprecationWarning)
         return tuple((unicode_type(k), v) for k, v in x.items())
 
     # only iterate once through the options.
@@ -207,11 +207,13 @@ class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
         self._options_values = tuple(i[1] for i in options)
         if self._initializing_traits_ is not True:
             if len(options) > 0:
-                # if the index is already 0, we set it to None so that setting it to 0
-                # will be recognized as a change (and pick up the new option value and label).
                 if self.index == 0:
-                    self.index = None
-                self.index = 0
+                    # Explicitly trigger the observers to pick up the new value and
+                    # label. Just setting the value would not trigger the observers
+                    # since traitlets thinks the value hasn't changed.
+                    self._notify_trait('index', 0, 0)
+                else:
+                    self.index = 0
             else:
                 self.index = None
 
@@ -242,7 +244,12 @@ class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
 
     @observe('value')
     def _propagate_value(self, change):
-        index = self._options_values.index(change.new) if change.new is not None else None
+        if change.new is None:
+            index = None
+        elif self.index is not None and self._options_values[self.index] == change.new:
+            index = self.index
+        else:
+            index = self._options_values.index(change.new)
         if self.index != index:
             self.index = index
 
@@ -254,7 +261,12 @@ class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
 
     @observe('label')
     def _propagate_label(self, change):
-        index = self._options_labels.index(change.new) if change.new is not None else None
+        if change.new is None:
+            index = None
+        elif self.index is not None and self._options_labels[self.index] == change.new:
+            index = self.index
+        else:
+            index = self._options_labels.index(change.new)
         if self.index != index:
             self.index = index
 
@@ -527,6 +539,13 @@ class _SelectionNonempty(_Selection):
             raise TraitError("Option list must be nonempty")
         return proposal.value
 
+    @validate('index')
+    def _validate_index(self, proposal):
+        if 0 <= proposal.value < len(self._options_labels):
+            return proposal.value
+        else:
+            raise TraitError('Invalid selection: index out of bounds')
+
 class _MultipleSelectionNonempty(_MultipleSelection):
     """Selection that is guaranteed to have an option available."""
 
